/* OpenCA OCSP responder
* (c) 2000-2006 by Massimiliano Pala and OpenCA Group
* All Rights Reserved
*
* ===================================================================
* Released under OpenCA LICENSE
*/

#ifndef HEADER_OPENCA_OCSPD_GENERAL_H
#define HEADER_OPENCA_OCSPD_GENERAL_H

#include "config.h"
#include "crypto.h"

#include <pthread.h>
#include "structs.h"

/* External Variables */
#if STDC_HEADERS
# include <string.h>
# include <stdio.h>
# include <stdlib.h>
#else
# ifndef HAVE_STRCHR
#  define strchr index
#  define strrchr rindex
# endif
char *strchr (), *strrchr ();
# ifndef HAVE_MEMCPY
#  define memcpy(d, s, n) bcopy ((s), (d), (n))
#  define memmove(d, s, n) bcopy ((s), (d), (n))
# endif
#endif

#include <syslog.h>
#include <ctype.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <sys/file.h>
#include <sys/times.h>

#include <sys/sem.h>
#include <sys/ipc.h>

#ifdef OCSPD_ARCH_SOLARIS
#include <fcntl.h>
#endif

#if defined (__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
	/* Ok, Semafores are correctly supported */
#elif defined (_SYS_SEM_H_)
	/* OpenBSD is not a GNU_LIBRARY but knows semaphores */
#else
	/* We should define here the structure */
	union semun {
             int val;                  /* value for SETVAL */
             struct semid_ds *buf;     /* buffer for IPC_STAT, IPC_SET */
             unsigned short *array;    /* array for GETALL, SETALL */
                                       /* Linux specific part: */
             struct seminfo *__buf;    /* buffer for IPC_INFO */
       };

#endif

#include <openssl/conf.h>
#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/bn.h>
#include <openssl/txt_db.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/objects.h>
#include <openssl/ocsp.h> 
#include <openssl/pem.h>

#ifdef HAVE_ENGINE
#include <openssl/engine.h>
#endif

#include <openssl/ocsp.h>

#define BASE_SECTION    	"ocspd"
#define CONFIG_FILE 		"ocspd.conf"
#define ENV_SECTION 		"conf_section"

#define ENV_DEFAULT_OCSPD	"default_ocspd"
#define ENV_OCSPD_DBMS		"dbms"
#define ENV_PRIVATE_KEY		"ocspd_key"
#define ENV_CERTIFICATE		"ocspd_certificate"
#define ENV_OCSPD_PORT		"port"
#define ENV_OCSPD_BIND		"bind"
#define ENV_OCSPD_CHILD		"threads_num"
#define ENV_OCSPD_QUEUE		"max_client_num"
#define ENV_OCSPD_MAXRS		"max_req_size"
#define ENV_OCSPD_MAX_NET_TIMEOUT	"max_timeout_secs"
#define ENV_OCSPD_MD		"md"
#define ENV_OCSPD_PIDFILE	"pidfile"
#define ENV_OCSPD_RESPONSE	"response"
#define ENV_OCSPD_RESPONSE_KEYID "ocsp_add_response_keyid"
#define ENV_OCSPD_RESPONSE_OTHER_CERTS "ocsp_add_response_certs"
#define ENV_OCSPD_RESPONSE_DAYS	"next_update_days"
#define ENV_OCSPD_RESPONSE_MINS	"next_update_mins"
#define ENV_OCSPD_CHROOT_DIR	"chroot_dir"

#define ENV_OCSPD_CA_SECTION	"ca"
#define ENV_OCSPD_CRL_URL	"crl_url"
#define ENV_OCSPD_CRL_ENTRY_DN	"crl_entry_dn"
#define ENV_OCSPD_CRL_ENTRY_ATTR "crl_entry_attribute"
#define ENV_OCSPD_CACERT_URL	"ca_url"
#define ENV_OCSPD_CACERT_ENTRY_DN "ca_entry_dn"
#define ENV_OCSPD_CACERT_ENTRY_ATTR "ca_entry_attribute"
#define ENV_OCSPD_CA_COMPROMISED "ca_compromised"

#define ENV_OCSPD_CRL_CHECK_VALIDITY "crl_check_validity"
#define ENV_OCSPD_CRL_RELOAD_EXPIRED "crl_reload_expired"
#define ENV_OCSPD_CRL_AUTO_RELOAD "crl_auto_reload"

#define ENV_OCSPD_USER		"user"
#define ENV_OCSPD_GROUP		"group"

#ifdef HAVE_ENGINE
#define ENV_OCSPD_ENGINE	"engine"
#define ENV_OCSPD_ENGINE_ID	"engine_id"
#define ENV_OCSPD_ENGINE_PRE	"engine_pre"
#define ENV_OCSPD_ENGINE_POST	"engine_post"
#endif

#define OCSP_REQ_CONTENT_TYPE	"application/ocsp-request"
#define OCSP_RESP_CONTENT_TYPE	"application/ocsp-response"

#define FORMAT_UNDEF    0
#define FORMAT_ASN1     1
#define FORMAT_TEXT     2
#define FORMAT_PEM      3
#define FORMAT_NETSCAPE 4
#define FORMAT_PKCS12   5
#define FORMAT_SMIME    6
#define FORMAT_ENGINE   7

#define APP_PASS_LEN    1024

typedef struct pw_cb_data
	{
	const void *password;
	const char *prompt_info;
	} PW_CB_DATA;

#define OCSP_CRL_PROTO_FILE		0
#define OCSP_CRL_PROTO_LDAP		1
#define OCSP_CRL_PROTO_HTTP		2

#define OCSP_DEFAULT_LDAP_PORT		389
#define OCSP_DEFAULT_HTTP_PORT		80

typedef struct url_data
	{
		/* Protocol, currently supported LDAP and FILE */
		int proto;

		/* Address or filename */
		char *addr;

		/* Communication Port (where supported by the protocol) */
		int port;

		/* Authentication (where supported by the protocol) */
		char *usr;
		char *pwd;

		/* LDAP specific */
		char *dn;
		char *attr;

		/* HTTP specific */
		char *path;
	} URL;

#define CRL_DEF_CHECK_INTERVAL	15

#define CRL_OK			1
#define	CRL_NOT_YET_VALID	2
#define	CRL_EXPIRED		3
#define	CRL_ERROR_NEXT_UPDATE	4
#define	CRL_ERROR_LAST_UPDATE	5
#define CRL_ERROR_UNKNOWN	10

typedef struct crl_data
	{
		/* CRL access method OCSP_CRL_METHOD_... */
		/* Filename */
		URL *url;

		X509_CRL *crl;
	} CRL_DATA;

typedef struct x509_crl_entry 
	{
		long reason;

		/* Serial Number of the entry */
		ASN1_INTEGER *serial;

		/* Revocation Time */
		ASN1_TIME *rev_time;

		/* Invalidity time, if reason in KeyTime or
		 * CAKeyTime  */
		ASN1_GENERALIZEDTIME *invalidity_time;

		/* Hold Instruction, if present */
		ASN1_OBJECT *hold_instr;

	} X509_CRL_ENTRY;

typedef struct ca_entry_certid
	{

		/* This is just the structure which we want to
		   memcmp - the len is needed as we don't know the
		   exact length of the passed hash (it is the upper
		   limit for the memcmp() */

		X509_ALGOR *hashAlgorithm;

		ASN1_OCTET_STRING *keyHash;
		ASN1_OCTET_STRING *nameHash;

	} CA_ENTRY_CERTID;

#define sk_CA_ENTRY_CERTID_new_null() SKM_sk_new_null(CA_ENTRY_CERTID)
#define sk_CA_ENTRY_CERTID_push(st, val) SKM_sk_push(CA_ENTRY_CERTID, (st), (val))
#define sk_CA_ENTRY_CERTID_pop(st) SKM_sk_pop(CA_ENTRY_CERTID, (st))
#define sk_CA_ENTRY_CERTID_value(st, i) SKM_sk_value(CA_ENTRY_CERTID, (st), (i))
#define sk_CA_ENTRY_CERTID_num(st) SKM_sk_num(CA_ENTRY_CERTID, (st))
#define sk_CA_ENTRY_CERTID_sort(st) SKM_sk_sort(CA_ENTRY_CERTID, (st))
#define sk_CA_ENTRY_CERTID_find(st) SKM_sk_find(CA_ENTRY_CERTID, (st))

/* List of available CAs */
typedef struct ca_list_st
	{
		/* CA ID - from section in config file */
		char *ca_id;

		/* CA Status - If compromised > 0 respond all revoked */
		int compromised;

		/* CA certificate */
		STACK_OF(X509) *cert;

		/* URL */
		URL *url;

		/* CRL URL */
		URL *crl_url;

		/* CRL data */
		X509_CRL *crl;

		/* Cert Identifier */
		STACK_OF(CA_ENTRY_CERTID) *cid;

		/* Pointer to the list of CRLs entries */
		STACK_OF(X509_REVOKED) *crl_list;

		/* X509 nextUpdate and lastUpdate */
		ASN1_TIME *nextUpdate;
		ASN1_TIME *lastUpdate;

		/* Options for auto reloading of CRL upon expiration */
		int crl_status;
		// int crl_reload_expired;

		/* Number of entries present in the list */
		unsigned long entries_num;

	} CA_LIST_ENTRY;

typedef struct ocspd_config {

	/* Configuration file name */
	char *cnf_filename;

	/* Verbose Flag */
	int verbose;
	int debug;

	/* CONF strucutre pointer */
	CONF * conf;

	int nmin;
	int ndays;
	int set_nextUpdate;

	int flags;

	/* Text db file */
	/* char *db_file; */
	CRL_DATA crl_data;

	/* User and Group the processes will run as */
	char *user;
	char *group;
	char *chroot_dir;

	/* Digest to be used */
	EVP_MD *digest;

	/* CA's  certificate */
	X509 *cacert;

	/* OCSP responder certificate and pkey */
	X509 *ocspd_cert;
	EVP_PKEY *ocspd_pkey;

	/* Other certificates to be added to the response, if any */
	STACK_OF(X509) *other_certs;

	/* CAs entry list */
	long ca_list_len;
	CA_LIST_ENTRY **ca_list;

	/* CRL validity period checking */
	int crl_check_validity;
	int crl_auto_reload;
	int crl_reload_expired;

	int current_crl_reload;
	int current_crl_check;
	int alarm_decrement;

	/* Network related */
	ssize_t max_req_size;
	unsigned int max_timeout_secs;

	/* Threads Management */
	int iget;
	int iput;

	int *clifd;

	int connfd;
	int listenfd;

	int nthreads;
	Thread *threads_list;


#ifdef HAVE_ENGINE
	char *engine_id;

	ENGINE *engine;

	STACK *pre_cmds;
	STACK *post_cmds;
#endif
} OCSPD_CONFIG;

#define CRL_REASON_UNSPECIFIED			0
#define CRL_REASON_KEY_COMPROMISE		1
#define CRL_REASON_CA_COMPROMISE		2
#define CRL_REASON_AFFILIATION_CHANGED		3
#define CRL_REASON_SUPERSEDED			4
#define CRL_REASON_CESSATION_OF_OPERATION	5
#define CRL_REASON_CERTIFICATE_HOLD		6
#define CRL_REASON_REMOVE_FROM_CRL		8
#define CRL_REASON_HOLD_INSTRUCTION		9
#define CRL_REASON_KEY_TIME			10
#define CRL_REASON_CA_KEY_TIME			11

#define SHARED_MEM_BLOCK_SIZE			8192

/*
static char *crl_reasons[] = {
        // CRL reason strings 
        "unspecified",
        "keyCompromise",
        "CACompromise",
        "affiliationChanged",
        "superseded",
        "cessationOfOperation",
        "certificateHold",
        "removeFromCRL",
        // Additional pseudo reasons
        "holdInstruction",
        "keyTime",
        "CAkeyTime"
};

#define NUM_REASONS (sizeof(crl_reasons) / sizeof(char *))
*/
#endif
